// ***********************************************************************************************************************
// ** File:         global.h                                                                                            **
// ** Title:        Some Globals                                                                                        **
// ** CPU:          ESP-8266                                                                                            **
// ** Created:      23.04.2016                                                                                          **
// ** Revised:      23.04.2016                                                                                          **
// ** Version:      1.0                                                                                                 **
// ** Note:                                                                                                             **
// **                                            ESP-01                                                                 **
// **                                    __________________________                                                     **
// **                                   |             ___________  |                                                    **
// **                         GPIO1 TXD | O O GND        |  _____| |                                                    **
// **                           CH_PD * | O O GPIO2      | |_____  |                                                    **
// **                               RST | O O GPIO0 **   |  _____| |                                                    **
// **                               VCC | O O RXD GPIO3    |_____  |                                                    **
// **                                   |__________________________|                                                    **
// **                                                                                                                   **
// **                              * Tie to VCC except for Power Down Mode                                              **
// **                             ** Ground on Power Up for Program Mode                                                **
// **                                                                                                                   **
// **  Flashsize 1M(128K SPIFFS)                                                                                        **
// ***********************************************************************************************************************

// INCLUDES **************************************************************************************************************
#include "Steckdose.h"                                                      // 
// ***********************************************************************************************************************

// INCLUDES **************************************************************************************************************
#define OFF                 0                                               // 
#define ON                  1                                               // 
// ***********************************************************************************************************************

// VARIABLES *************************************************************************************************************
unsigned char           switch1       = OFF;                                //
unsigned int            button_pressed = 0;                                 // 

const int               wlan_led      = 4;                                  // GPIO 4
const int               button        = 14;                                 // GPIO 14
const int               relay_on      = 12;                                 // GPIO 12
const int               relay_off     = 5;                                  // GPIO 5

unsigned long           prev_millis   = 0;                                  //
unsigned char           temp_day      = 0;                                  //
unsigned char           temp_minute   = 0;                                  //
unsigned char           temp_switch1  = 0;                                  // 
long                    rssi          = 0;                                  //
long                    avg_rssi      = 0;                                  //
unsigned int            count         = 0;                                  //


unsigned long           recon_count   = 0;

// ***********************************************************************************************************************

// FUNCTION **************************************************************************************************************
void setup ( void )                                                         //
{                                                                           //
  Serial.begin(115200);                                                     //
  Serial.println();                                                         //

  gotIpEventHandler = WiFi.onStationModeGotIP([](const WiFiEventStationModeGotIP& event)//
  {                                                                         //
    digitalWrite        ( wlan_led,   ON );                                 // Ausgang auf 0
    Serial.print("Station connected, IP: ");                                //
    Serial.println(WiFi.localIP());                                         //
    recon_count++;
  });                                                                       //

  disconnectedEventHandler = WiFi.onStationModeDisconnected([](const WiFiEventStationModeDisconnected& event)//
  {                                                                         //
    digitalWrite        ( wlan_led,   OFF );                                // Ausgang auf 0
    Serial.println("Station disconnected");                                 //
    WiFiStart ( );
  });                                                                       //

  prev_millis         = millis ( );                                         //
  eep_init            ( );                                                  // 
  read_eeprom_config  ( );                                                  // 
  
  pinMode             ( wlan_led,   OUTPUT  );                              // DDR Register
  pinMode             ( button,     INPUT_PULLUP  );                        // DDR Register
  pinMode             ( relay_on,   OUTPUT  );                              // DDR Register
  pinMode             ( relay_off,  OUTPUT  );                              // DDR Register

  digitalWrite        ( wlan_led,   OFF );                                  // Ausgang auf 0
  digitalWrite        ( button,     OFF );                                  // Ausgang auf 0
  digitalWrite        ( relay_on,   ON  );                                  // Ausgang auf 1
  digitalWrite        ( relay_off,  ON  );                                  // Ausgang auf 1
  switch1             = OFF;                                                //

  dhcp_service    = EEPROM.read ( EE_DHCP_SERV );                           //
  if ( dhcp_service == OFF )                                                //
  {                                                                         //
    WiFi.config         ( ip_adress, dns_server, gateway, subnet_mask );    //
  }                                                                         //

//  WiFi.persistent     (false);
//  WiFi.mode           (WIFI_OFF);   // this is a temporary line, to be removed after SDK update to 1.5.4
//  WiFi.mode           (WIFI_STA);  
//
//  WiFi.mode           ( WIFI_STA );                                         //
  WiFiStart           ( );                                                  //
  espota              ( );                                                  //
  udp_start           ( );                                                  //
  
  SPIFFS.begin        ( );                                                  // 
  mqtt_init           ( );                                                  // 
  http_server_init    ( );                                                  // 
  
  if ( MDNS.begin ( dev_name ) );                                           // 
  send_ntp_packet     ( );                                                  //
}                                                                           //
// ***********************************************************************************************************************

// FUNCTION **************************************************************************************************************
void loop ( void )                                                          //
{                                                                           // 
  mqtt_connect ();                                                          // 
  if ( ( millis ( ) - prev_millis ) >= 1000 )                               // Every Second
  {                                                                         // 
    timestamp++;                                                            // 
    timestamp_decode ( );                                                   // 
    prev_millis += 1000;                                                    // 
    rssi = WiFi.RSSI();                                                     // 
    avg_rssi += rssi;                                                       // 
  }                                                                         // 
  if ( minute       != temp_minute )                                        // Every Minute
  {                                                                         //
    
    Serial.print    ( "Time: ");                                            //
    Serial.print    ( hour );                                               //
    Serial.print    ( ": ");                                                //
    Serial.println  ( minute );                                             //
    Serial.print    ( "Reconnect: " );                                      //
    Serial.println  ( recon_count   );                                      //
    Serial.print    ( "WLAN Status: ");                                     //
    Serial.println  ( WiFi.status());                                       //
    Serial.print    ( "Station connected, IP: ");                           //
    Serial.println  ( WiFi.localIP());                                      //
    

    
    if ( ntp_flag    != NTP_ANSWER )                                        // 
    {                                                                       // 
      send_ntp_packet ( );                                                  // 
    }                                                                       // 
    mqtt_send_state ( );                                                    //
    avg_rssi = 0;                                                           //
    temp_minute = minute;                                                   //
  }                                                                         //
  if ( day          != temp_day )                                           // Every Day
  {                                                                         // 
    mqtt_send_date  ( );                                                    //
    send_ntp_packet ( );                                                    // 
    temp_day = day;                                                         // 
  }                                                                         // 
  if ( digitalRead  ( button ) == 0 )                                       //
  {                                                                         //
    if ( button_pressed == 0 )                                              //
    {                                                                       //
      if ( switch1 == OFF )                                                 //
      {                                                                     //
        digitalWrite  ( relay_on, OFF );                                    //
        delay         ( 5 );                                                //
        digitalWrite  ( relay_on, ON  );                                    //
        switch1       = ON;                                                 //
      }                                                                     //
      else                                                                  //
      {                                                                     //
        digitalWrite  ( relay_off,  OFF );                                  //
        delay         ( 5 );                                                //
        digitalWrite  ( relay_off,  ON  );                                  //
        switch1       = OFF;                                                //
      }                                                                     //
      button_pressed = 1;                                                   //
    }                                                                       //
  }                                                                         //
  if ( digitalRead  ( button ) == 1 )                                       //
  {                                                                         //
    button_pressed = 0;                                                     //
  }                                                                         //
  yield();                                                                  // 
  if ( temp_switch1 != switch1  )                                           //
  {                                                                         // 
    temp_switch1 = switch1;                                                 //
    mqtt_send_switch  ( );                                                  // 
  }                                                                         // 
  yield();                                                                  // 
  server.handleClient();                                                    // 
  yield();                                                                  // 
  ArduinoOTA.handle();                                                      // 
  yield();                                                                  // 
  ntp_receiver ( );                                                         // 
  yield();                                                                  // 
}                                                                           //
// ***********************************************************************************************************************

// FUNCTION **************************************************************************************************************
void WiFiStart ( )                                                            // Connect to WiFi network
{                                                                             // 
  WiFi.persistent     ( false );                                              // 
  WiFi.mode           ( WIFI_OFF );                                           // this is a temporary line, to be removed after SDK update to 1.5.4
  WiFi.mode           ( WIFI_STA );                                           // 
  WiFi.begin          ( ssid, password );                                     // 
}                                                                             // 
// ***********************************************************************************************************************
