// ***********************************************************************************************************************
// ** File:         eep.h                                                                                               **
// ** Title:        Functions for EEProm                                                                                **
// ** CPU:          ESP-8266                                                                                            **
// ** Created:      15.12.2017                                                                                          **
// ** Revised:      15.12.2017                                                                                          **
// ** Version:      1.0                                                                                                 **
// ** Note:                                                                                                             **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// ***********************************************************************************************************************

// INCLUDES **************************************************************************************************************
#include "eep.h"                                                              // 
// ***********************************************************************************************************************

// VARIABLES *************************************************************************************************************
char                    ssid          [50];                                   //
char                    password      [50];                                   //

unsigned char           dhcp_service  = ON;                                   //
char                    dev_name      [50];                                   //
IPAddress               ip_adress;                                            //
IPAddress               subnet_mask;                                          //
IPAddress               dns_server;                                           //
IPAddress               gateway;                                              //
char                    ntp_server    [50];                                   //
char                    msg           [50];                                   //
// ***********************************************************************************************************************

// FUNCTION **************************************************************************************************************
void eep_init ( void )                                                        //
{                                                                             //
	char                ee_test[3];                                             //
	EEPROM.begin        ( 512 );                                                //
  eep_read_string     ( ee_test,     EE_EEPROM_OK );                          //
  if  ( String(ee_test) != String("OK") )                                     //
  {                                                                           //
    save_default_config ( );                                                  //
  }                                                                           //
}                                                                             //
// ***********************************************************************************************************************

// FUNCTION **************************************************************************************************************
void save_default_config (  )                                                 //
{                                                                             //
  snprintf              ( ssid,       50,   "Deine SSID"  );                      //
  snprintf              ( password,   50,   "Dein AP Passwort"  );      //

  dhcp_service          = OFF;                                                //
  snprintf              ( dev_name,   50,   "Steckdose 000"  );               //
  ip_adress             = { 192, 168, 1, 10  };                               //
  subnet_mask           = { 255, 255, 0, 0   };                               //
  dns_server            = { 192, 168, 0, 1   };                               //
  gateway               = { 192, 168, 0, 1   };                               //
  snprintf              ( ntp_server, 50,   "192.168.0.1"  );                 //

  mqtt_service          = OFF;                                                //
  snprintf              ( mqtt_broker, 50,  "192.168.1.1" );                  //
  snprintf              ( mqtt_user,  50,   "mqtt user"     );                //
  snprintf              ( mqtt_pass,  50,   "mqtt pass"     );                //
  snprintf              ( mqtt_topic, 100,  "/Raum/Steckdose000"   );         //

  eep_write_string      ( ssid,         EE_SSID );                            //
  eep_write_string      ( password,     EE_WL_PWD );                          //

  EEPROM.write          ( EE_DHCP_SERV, dhcp_service );                       //
  eep_write_string      ( dev_name,     EE_DEV_NAME );                        //
  eep_write_ip          ( ip_adress,    EE_IP_ADR );                          //
  eep_write_ip          ( subnet_mask,  EE_SUB_ADR );                         //
  eep_write_ip          ( dns_server,   EE_DNS_ADR );                         //
  eep_write_ip          ( gateway,      EE_GW_ADR );                          //
  eep_write_string      ( ntp_server,   EE_NTP_ADR );                         //

  EEPROM.write          ( EE_MQTT_SERV, mqtt_service );                       //
  eep_write_string      ( mqtt_broker,  EE_MQTT_BROKER );                     //
  eep_write_string      ( mqtt_user,    EE_MQTT_USER );                       //
  eep_write_string      ( mqtt_pass,    EE_MQTT_PWD );                        //
  eep_write_string      ( mqtt_topic,   EE_MQTT_TOPIC );                      //

  eep_write_string      ( "OK",         EE_EEPROM_OK );                       // 
  EEPROM.commit();                                                            //
}                                                                             //
// ***********************************************************************************************************************

// FUNCTION **************************************************************************************************************
IPAddress eep_read_ip      ( unsigned int eep_address )                       //
{                                                                             //
  IPAddress temp_ip;                                                          // 
  temp_ip[0] = EEPROM.read  ( eep_address    );                               // 
  temp_ip[1] = EEPROM.read  ( eep_address + 1 );                              // 
  temp_ip[2] = EEPROM.read  ( eep_address + 2 );                              // 
  temp_ip[3] = EEPROM.read  ( eep_address + 3 );                              // 
  return temp_ip;                                                             // 
}                                                                             //
// ***********************************************************************************************************************

// FUNCTION **************************************************************************************************************
void eep_write_ip      ( IPAddress temp_ip, unsigned int eep_address )        //
{                                                                             //
  EEPROM.write  ( eep_address,    (uint8_t)temp_ip[0] );                      //
  EEPROM.write  ( eep_address + 1, (uint8_t)temp_ip[1] );                     //
  EEPROM.write  ( eep_address + 2, (uint8_t)temp_ip[2] );                     //
  EEPROM.write  ( eep_address + 3, (uint8_t)temp_ip[3] );                     //
  EEPROM.commit ( );                                                          // 
}                                                                             //
// ***********************************************************************************************************************

// FUNCTION **************************************************************************************************************
unsigned int eep_read_string  ( char *buffer_data, unsigned int eep_address ) //
{                                                                             //
  unsigned int  zgr     = 0;                                                  // Zeiger auf 0
  unsigned char zeichen = 0xff;                                               // Zeichen auf FF damit while bedingung nicht direkt erf�llt ist
  while (zeichen != 0x00)                                                     // Auf NULL Zeichen pr�fen
  {                                                                           //
    zeichen = EEPROM.read  ( eep_address + zgr );                             //
    buffer_data[zgr] = zeichen;                                               //
    zgr++;                                                                    //
  }                                                                           //
  return zgr - 1;                                                             // Stringl�nge zur�ckgeben
}                                                                             //
// ***********************************************************************************************************************

// EEProm Daten String schreiben *****************************************************************************************
void eep_write_string         ( char *eep_data, unsigned int eep_address )    // 
{                                                                             // 
  unsigned int i;                                                             // 
  for ( i = 0 ; i < strlen( eep_data ) + 1 ; i++ )                            // 
  {                                                                           // 
    EEPROM.write  ( eep_address + i, eep_data[i] );                           // 
  }                                                                           // 
  EEPROM.commit();                                                            // 
}                                                                             // 
// ***********************************************************************************************************************

// FUNCTION **************************************************************************************************************
void read_eeprom_config ()                                                    //
{                                                                             //
  IPAddress       temp_ip;                                                    //
  char            temp_string      [100];                                     //
  unsigned char   temp_val;                                                   //

  eep_read_string ( ssid,         EE_SSID );                                  //
  eep_read_string ( password,     EE_WL_PWD );                                //

  dhcp_service    = EEPROM.read ( EE_DHCP_SERV );                             //
  eep_read_string ( dev_name,     EE_DEV_NAME );                              //
  ip_adress       = eep_read_ip ( EE_IP_ADR );                                //
  subnet_mask     = eep_read_ip ( EE_SUB_ADR );                               //
  dns_server      = eep_read_ip ( EE_DNS_ADR );                               //
  gateway         = eep_read_ip ( EE_GW_ADR );                                //
  eep_read_string ( ntp_server,   EE_NTP_ADR );                               //

  mqtt_service    = EEPROM.read ( EE_MQTT_SERV );                             //
  eep_read_string ( mqtt_broker,  EE_MQTT_BROKER );                           //
  eep_read_string ( mqtt_user,    EE_MQTT_USER );                             //
  eep_read_string ( mqtt_pass,    EE_MQTT_PWD );                              //
  eep_read_string ( mqtt_topic,   EE_MQTT_TOPIC );                            //
}                                                                             //
// ***********************************************************************************************************************

// HTTP Ascii IP in eine "richtige" IP wandeln ***************************************************************************
IPAddress http_asc_to_ip ( String ascip )                                     // In http kommt die IP so an :
{                                                                             //
  IPAddress result = {0, 0, 0, 0};                                            //
  char      temp_str[16];                                                     // 192.168.100.123/0 = 16 Bytes
  ascip.toCharArray ( temp_str, 16 );                                         // Aus String Char Array machen
  unsigned char i = 0;                                                        //
  while (i < 16 && temp_str[i] )                                              // Wenn schleife 4 mal durchlaufen ist dann
  {                                                                           // ist die IP adresse komplett
    if  (                                                                     // 
          ( temp_str[i] == '.' ) ||                                           // 
          ( temp_str[i] >= '0' ) ||                                           // 
          ( temp_str[i] <= '9' )                                              // 
        )                                                                     // Ip darf nur aus Ziffern und Punkten zusammengesetzt sein
    {                                                                         // 
      i++;                                                                    // Alles Ok weitermachen
    }                                                                         // 
    else                                                                      // 
    {                                                                         // 
      return result;                                                          // IP fehlerhaft 0 zurückgeben
    }                                                                         // 
  }                                                                           // 
  if (temp_str[i] == '\0')                                                    // 
  {                                                                           //
    unsigned char octett = 0, oct_val = 0;                                    // 
    i = 0;                                                                    // 
    while ( !(temp_str[i] == '\0') &&  octett < 4 )                           //
    {                                                                         // 
      if ( temp_str[i] == '.' )                                               // 
      {                                                                       // 
        if ( oct_val > 255 )                                                  //  
        {                                                                     // 
          result = {0, 0, 0, 0};                                              // 
          return result;                                                      // IP fehlerhaft 0 zurückgeben
        }                                                                     // 
        else                                                                  // 
        {                                                                     // 
          result[octett] = (byte)oct_val;                                     // 
          octett++;                                                           // 
          oct_val = 0;                                                        // 
        }                                                                     // 
      }                                                                       // 
      else                                                                    // 
      {                                                                       // 
        oct_val = (oct_val * 10) + (temp_str[i] - '0');                       // 
      }                                                                       // 
      i++;                                                                    // 
    }                                                                         // 
    if ((oct_val > 255) || (octett > 3))                                      // You can't have IP address segments that don't fit in a byte, or more than four segments
    {                                                                         // 
      result = {0, 0, 0, 0};                                                  // 
      return result;                                                          // IP fehlerhaft 0 zurückgeben
    }                                                                         // 
    else                                                                      // 
    {                                                                         // 
      result[octett] = (byte)oct_val;                                         // 
      return result;                                                          // 
    }                                                                         // 
  }                                                                           // 
  else                                                                        // 
  {                                                                           // 
    result = {0, 0, 0, 0};                                                    // 
    return result;                                                            // IP fehlerhaft 0 zurückgeben
  }                                                                           // 
}                                                                             //
// ***********************************************************************************************************************
