// ***********************************************************************************************************************
// ** File:         http.ino                                                                                            **
// ** Title:        Functions for HTTP-Protocol                                                                         **
// ** CPU:          ESP-8266                                                                                            **
// ** Created:      24.12.2017                                                                                          **
// ** Revised:      24.12.2017                                                                                          **
// ** Version:      1.0                                                                                                 **
// ** Note:                                                                                                             **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// ***********************************************************************************************************************

// INCLUDES **************************************************************************************************************
#include "http.h"                                                             // 
// ***********************************************************************************************************************

// DEFINES **************************************************************************************************************

// ***********************************************************************************************************************

// VARIABLES *************************************************************************************************************

// ***********************************************************************************************************************

// FUNCTION **************************************************************************************************************
void http_server_init ( void )                                                // 
{                                                                             // 
  server.serveStatic  ( "/index.html",   SPIFFS, "/index.html"  );            // 
  server.serveStatic  ( "/",             SPIFFS, "/index.html"  );            // 
  server.serveStatic  ( "/mqtt.html",    SPIFFS, "/mqtt.html"   );            // 
  server.serveStatic  ( "/setup.html",   SPIFFS, "/setup.html"  );            // 
  server.serveStatic  ( "/wlan.html",    SPIFFS, "/wlan.html"   );            // 
  server.serveStatic  ( "/style.css",    SPIFFS, "/style.css"   );            // 
  server.serveStatic  ( "/java.js",      SPIFFS, "/java.js"     );            // 
  server.serveStatic  ( "/mqtt.js",      SPIFFS, "/mqtt.js"     );            // 
  server.serveStatic  ( "/wlan.js",      SPIFFS, "/wlan.js"     );            // 
  server.serveStatic  ( "/setup.js",     SPIFFS, "/setup.js"    );            // 
  server.serveStatic  ( "/favicon.ico",  SPIFFS, "/favicon.ico" );            // 
  http_dyndata_0      ( );                                                    //
  http_dyndata_1      ( );                                                    //
  http_dyndata_2      ( );                                                    //
  http_dyndata_3      ( );                                                    //
  server.onNotFound   ( handleNotFound );                                     //
  server.begin        ( );                                                    //
}                                                                             // 
// ***********************************************************************************************************************

// FUNCTION **************************************************************************************************************
void http_dyndata_0 ( void )                                                  // 
{                                                                             // 
  server.on ( "/dyndata.html",  []()                                          //
  {                                                                           //
    String switch1_state = server.arg("switch1");                             //
    char temp[300];                                                           //
    if ( switch1_state == "1" )                                               //
    {                                                                         //
      digitalWrite  ( relay_on, OFF );                                        //
      delay         ( 5 );
      digitalWrite  ( relay_on, ON  );                                        //
      switch1       = ON;                                                     //
    }                                                                         //
    else if ( switch1_state == "0" )                                          //
    {                                                                         //
      digitalWrite  ( relay_off,  OFF );                                      //
      delay         ( 5 );
      digitalWrite  ( relay_off,  ON  );                                      //
      switch1       = OFF;                                                    //
    }                                                                         //
    snprintf  (                                                               //
                temp,                                                         //
                300,                                                          //
                "[\"0\","                                                     //
                "\"%s %02d.%s %d\","                                          //
                "\"%02d:%02d:%02d\","                                         //
                "\"%d\","                                                     //
                "\"%d\","                                                     //
                "\"%s\"]",                                                    //
                Day_to_wdays[wday], day, Month_to_string[month - 1], year,    //
                hour, minute, second,                                         //
                switch1,                                      //
                rssi,                                                         //
                dev_name                                                      //
              );                                                              //
    server.send ( 200, "application/json", temp);                             //
  } );                                                                        //
}                                                                             // 
// ***********************************************************************************************************************

// FUNCTION **************************************************************************************************************
void http_dyndata_1 ( void )                                                  // 
{                                                                             // 
  server.on ( "/mqttdata.html", []()                                          //
  {                                                                           // 
    char    temp[300];                                                        // 
    String  mqttserv_param  = server.arg("mqttserv");                         //
    String  mqttadr_param   = server.arg("mqttadr");                          //
    String  mqttusr_param   = server.arg("mqttuser");                         //
    String  mqttpwd_param   = server.arg("mqttpass");                         //
    String  mqtttopic_param = server.arg("mqtttopic");                        //

    if ( mqttserv_param != "" )                                               // 
    {                                                                         // 
      if ( mqttserv_param == "0" )                                            // MQTT Off
      {                                                                       // 
        mqtt_service  = OFF;                                                  //
        EEPROM.write          ( EE_MQTT_SERV, mqtt_service );                 //
        client.disconnect     ( );                                            //
      }                                                                       // 
      if ( mqttserv_param == "1" )                                            // MQTT On
      {                                                                       // 
        mqtt_service  = ON;                                                   //
        EEPROM.write          ( EE_MQTT_SERV, mqtt_service );                 //
        mqtt_init           ( );                                                    // 
      }                                                                       // 
    }                                                                         // 
    
    if ( !(mqttadr_param == "") )                                             // 
    {                                                                         // 
      mqttadr_param.toCharArray ( mqtt_broker, 50 );                          //
      eep_write_string  ( mqtt_broker,  EE_MQTT_BROKER );                     //
    }                                                                         // 
    if ( !(mqttusr_param == "") )                                             // 
    {                                                                         // 
      mqttusr_param.toCharArray ( mqtt_user, 50 );                            //
      eep_write_string  ( mqtt_user,    EE_MQTT_USER );                       //
    }                                                                         // 
    if ( !(mqttpwd_param == "") )                                             //
    {                                                                         // 
      mqttpwd_param.toCharArray ( mqtt_pass, 50 );                            //
      eep_write_string  ( mqtt_pass,    EE_MQTT_PWD );                        //
    }                                                                         // 
    if ( !(mqtttopic_param == "") )                                           // 
    {                                                                         // 
      mqtttopic_param.toCharArray ( mqtt_topic, 100 );                        //
      eep_write_string  ( mqtt_topic,   EE_MQTT_TOPIC );                      //
    }                                                                         // 
    snprintf (                                                                // 
      temp,                                                                   // 
      300,                                                                    // 
      "[\"1\","                                                               // 
      "\"%d\","                                                               // 
      "\"%s\","                                                               // 
      "\"%s\","                                                               // 
      "\"%s\","                                                               // 
      "\"%s\"]"                                                               // 
      , mqtt_service                                                          // 
      , mqtt_broker                                                           // 
      , mqtt_user                                                             // 
      , mqtt_pass                                                             // 
      , mqtt_topic                                                            // 
    );                                                                        // 
    server.send ( 200, "application/json", temp);                             //
  } );                                                                        // 
}                                                                             // 
// ***********************************************************************************************************************

// FUNCTION **************************************************************************************************************
void http_dyndata_2 ( void )                                                  // 
{                                                                             // 
  server.on ( "/setupdata.html", []()                                         //
  {                                                                           // 
    char    temp[300];                                                        // 
    String  name_param  = server.arg("hostname");                             //
    String  dhcp_param  = server.arg("dhcp");                                 //
    String  ip_param    = server.arg("ipadr");                                //
    String  sub_param   = server.arg("subadr");                               //
    String  dns_param   = server.arg("dnsadr");                               //
    String  gw_param    = server.arg("gwadr");                                //
    String  ntp_param   = server.arg("ntpadr");                               //

    if ( dhcp_param != "" )                                                   // 
    {                                                                         // 
      if ( dhcp_param == "0" )                                                // MQTT Off
      {                                                                       // 
        dhcp_service  = OFF;                                                  //
        WiFi.disconnect       ( );
        EEPROM.write          ( EE_DHCP_SERV, dhcp_service );                 //
        WiFi.config           ( ip_adress, dns_server, gateway, subnet_mask );//
//        WiFi.mode             ( WIFI_STA );                                   //
        WiFiStart             ( );                                            //
      }                                                                       // 
      if ( dhcp_param == "1" )                                                // MQTT On
      {                                                                       // 
        dhcp_service  = ON;                                                   //
        WiFi.disconnect       ( );
        EEPROM.write          ( EE_DHCP_SERV, dhcp_service );                 //
//        WiFi.mode             ( WIFI_STA );                                   //
        WiFiStart             ( );                                            //
      }                                                                       // 
    }                                                                         // 
    if ( name_param != "" )                                                   // 
    {                                                                         // 
      name_param.toCharArray ( dev_name, 50 );                                //
      eep_write_string  ( dev_name,     EE_DEV_NAME );                        //
    }                                                                         // 
    if ( ip_param   != "" )                                                   //
    {                                                                         // 
      ip_adress = http_asc_to_ip ( ip_param );                                //
      eep_write_ip      ( ip_adress,    EE_IP_ADR );                          //
    }                                                                         // 
    if ( sub_param  != "" )                                                   //
    {                                                                         // 
      subnet_mask = http_asc_to_ip ( sub_param );                             //
      eep_write_ip      ( subnet_mask,  EE_SUB_ADR );                         //
    }                                                                         // 
    if ( dns_param  != "" )                                                   // 
    {                                                                         // 
      dns_server = http_asc_to_ip ( dns_param );                              //
      eep_write_ip      ( dns_server,   EE_DNS_ADR );                         //
    }                                                                         // 
    if ( gw_param   != "" )                                                   //
    {                                                                         // 
      gateway = http_asc_to_ip ( gw_param );                                  //
      eep_write_ip      ( gateway,      EE_GW_ADR );                          //
    }                                                                         // 
    if ( ntp_param  != "" )                                                   //
    {                                                                         // 
      ntp_param.toCharArray ( ntp_server, 50 );                               //
      eep_write_string  ( ntp_server,   EE_NTP_ADR );                         //
      send_ntp_packet ( );                                                    // 
    }                                                                         // 
    WiFi.config ( ip_adress, dns_server, gateway, subnet_mask );              //

    String  esp_mac    = WiFi.macAddress();                                   // 
    char    mac_addr [20];                                                    //String  mac_addr    = WiFi.macAddress();
    esp_mac.toCharArray ( mac_addr, 20 );

    snprintf  (                                                                // 
                temp,                                                                   // 
                300,                                                                    // 
                "[\"2\","                                                               // 
                "\"%s\","                                                               // 
                "\"%d\","                                                               // 
                "\"%d.%d.%d.%d\","                                                      // 
                "\"%d.%d.%d.%d\","                                                      // 
                "\"%d.%d.%d.%d\","                                                      // 
                "\"%d.%d.%d.%d\","                                                      // 
                "\"%s\","                                                               // 
                "\"%s\"]"                                                               // 
                , dev_name                                                              // 
                , dhcp_service                                                          // 
                , ip_adress[0]   , ip_adress[1]   , ip_adress[2]   , ip_adress[3]       // 
                , subnet_mask[0] , subnet_mask[1] , subnet_mask[2] , subnet_mask[3]     // 
                , dns_server[0]  , dns_server[1]  , dns_server[2]  , dns_server[3]      // 
                , gateway[0]     , gateway[1]     , gateway[2]     , gateway[3]         // 
                , ntp_server                                                            // 
                , mac_addr
              );                                                                        // 
    server.send ( 200, "application/json", temp);                             //
  } );                                                                        // 
}                                                                             // 
// ***********************************************************************************************************************

// FUNCTION **************************************************************************************************************
void http_dyndata_3 ( void )                                                  // 
{                                                                             // 
  server.on ( "/wlandata.html", []()                                          //
  {                                                                           // 
    char temp[300];                                                           // 
    String  ssid_param  = server.arg("ssid");                                 // 
    String  wpwd_param  = server.arg("wlanpass");                             // 
    if ( ssid_param != "" )                                                   // 
    {                                                                         // 
      ssid_param.toCharArray ( ssid, 50 );                                    // 
      eep_write_string  ( ssid,    EE_SSID );                                 //
      WiFiStart();                                                            // 
    }                                                                         // 
    if ( wpwd_param != "" )                                                   // 
    {                                                                         // 
      wpwd_param.toCharArray ( password, 50 );                                // 
      eep_write_string  ( password,    EE_WL_PWD );                           //
      WiFiStart();                                                            // 
    }                                                                         // 
    snprintf  (                                                               // 
                temp,                                                         // 
                300,                                                          // 
                "[\"3\","                                                     // 
                "\"%s\","                                                     // 
                "\"%s\"]"                                                     // 
                , ssid                                                        // 
                , password                                                    // 
              );                                                              // 
    server.send ( 200, "application/json", temp);                             //
  } );                                                                        // 
}                                                                             // 
// ***********************************************************************************************************************

// FUNCTION **************************************************************************************************************
void handleNotFound ( )                                                       // 
{                                                                             // 
  String message = "File Not Found\n\n";                                      // 
  message += "URI: ";                                                         // 
  message += server.uri();                                                    // 
  message += "\nMethod: ";                                                    // 
  message += ( server.method() == HTTP_GET ) ? "GET" : "POST";                // 
  message += "\nArguments: ";                                                 // 
  message += server.args();                                                   // 
  message += "\n";                                                            // 
  for ( uint8_t i = 0; i < server.args(); i++ )                               // 
  {                                                                           // 
    message += " " + server.argName ( i ) + ": " + server.arg ( i ) + "\n";   // 
  }                                                                           // 
  server.send ( 404, "text/plain", message );                                 // 
}                                                                             // 
// ***********************************************************************************************************************
