// ***********************************************************************************************************************
// ** File:         mqtt.ino                                                                                            **
// ** Title:        Functions for MQTT-Protocol                                                                         **
// ** CPU:          ESP-8266                                                                                            **
// ** Created:      23.12.2017                                                                                          **
// ** Revised:      23.12.2017                                                                                          **
// ** Version:      1.0                                                                                                 **
// ** Note:                                                                                                             **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// ***********************************************************************************************************************

// INCLUDES **************************************************************************************************************
#include "mqtt.h"                                                             // 
// ***********************************************************************************************************************

// DEFINES **************************************************************************************************************

// ***********************************************************************************************************************

// VARIABLES *************************************************************************************************************
unsigned char           mqtt_service  = ON;                                   //
char                    mqtt_broker   [50];                                   //
char                    mqtt_user     [50];                                   //
char                    mqtt_pass     [50];                                   //
char                    mqtt_topic    [100];                                  //
unsigned long           lastReconnectAttempt  = 0;                            // 
// ***********************************************************************************************************************

// FUNCTION **************************************************************************************************************
void mqtt_init  ( void )                                                      //
{                                                                             // 
  client.setServer    ( mqtt_broker, 1883 );                                  // 
  client.setCallback  ( callback );                                           // 
  lastReconnectAttempt = 0;                                                   // 
  char pwm1_topic[50];                                                        //
  snprintf (pwm1_topic, 50, "%s/switch", mqtt_topic);                      //
  //snprintf (pwm1_topic, 50, "%s/switch_in", mqtt_topic);                      //
  client.subscribe    ( pwm1_topic );                                         //
}                                                                             // 
// ***********************************************************************************************************************

// FUNCTION **************************************************************************************************************
void    mqtt_connect  ( void )                                                //
{                                                                             //
  if ( !client.connected  ( ) && mqtt_service == ON )                         // Check if MQTT Server is unreachable
  {                                                                           // 
    unsigned long now = millis ( );                                           // 
    if ( now - lastReconnectAttempt > 5000 )                                  //
    {                                                                         // 
      lastReconnectAttempt = now;                                             // Attempt to reconnect
      if ( reconnect ( ) )                                                    // 
      {                                                                       // 
        lastReconnectAttempt = 0;                                             // 
      }                                                                       // 
    }                                                                         // 
  }                                                                           // 
  else                                                                        // Client connected
  {                                                                           // 
    client.loop ( );                                                          // 
  }                                                                           // 
}                                                                             //
// ***********************************************************************************************************************

// FUNCTION **************************************************************************************************************
void mqtt_send_state  ( void )                                                //
{                                                                             //
  if (client.connected()  && mqtt_service == ON)                              //
  {                                                                           //
    char rssi_topic[50];                                                      //
    char switch1_topic[50];                                                   //
    snprintf ( rssi_topic, 50, "%s/rssi", mqtt_topic);                        //
    snprintf ( msg, 50, "%d", ( avg_rssi / 60 ) );                            //
    client.publish( rssi_topic, msg);                                         //
    snprintf (switch1_topic, 50, "%s/switch", mqtt_topic);                    //
    snprintf (msg, 50, "%d", temp_switch1);                                   //
    client.publish( switch1_topic, msg);                                      // 
  }                                                                           //
}                                                                             //
// ***********************************************************************************************************************

// FUNCTION **************************************************************************************************************
void mqtt_send_date   ( void )                                                //
{                                                                             //
  if ( client.connected ( )  && mqtt_service == ON )                          // 
    {                                                                         // 
      char date_topic[50];                                                    // 
      snprintf ( date_topic, 50, "%s/date", mqtt_topic);                      // 
      snprintf ( msg, 50, "%s %02d.%s %d", Day_to_wdays[wday], day, Month_to_string[month - 1], year );//
      client.publish(date_topic, msg);                                        // 
    }                                                                         // 
}                                                                             //
// ***********************************************************************************************************************

// FUNCTION **************************************************************************************************************
void mqtt_send_switch   ( void )                                              //
{                                                                             //
  if ( client.connected ()  && mqtt_service == ON )                           //
    {                                                                         //
      char switch1_topic[50];                                                 //
      temp_switch1 = switch1;                                                 //
      snprintf (switch1_topic, 50, "%s/switch", mqtt_topic);                  //
      snprintf (msg, 50, "%d", temp_switch1);                                 //
      client.publish( switch1_topic, msg);                                    // 
    }                                                                         //
}                                                                             //
// ***********************************************************************************************************************

// FUNCTION **************************************************************************************************************
boolean reconnect ( void )                                                    // 
{                                                                             // 
  if ( client.connect ( dev_name )  && mqtt_service == ON)                    // 
  {                                                                           // 
    client.publish    ( mqtt_topic, "Device is alive" );                      // Once connected, publish an announcement...
    char switch_topic[50];                                                    // 
    //snprintf ( switch_topic, 50, "%s/switch_in", mqtt_topic);                 // 
    snprintf ( switch_topic, 50, "%s/switch", mqtt_topic);                 // 
    client.subscribe  ( switch_topic );                                       // ... and resubscribe
  }                                                                           // 
  return client.connected ( );                                                // 
}                                                                             // 
// ***********************************************************************************************************************

// FUNCTION **************************************************************************************************************
void callback(char* topic, byte* payload, unsigned int length)                //
{                                                                             // 
  String mqtt_in_data;                                                        //
  for (int i = 0; i < length; i++)                                            //
  {                                                                           //
    mqtt_in_data += (char)payload[i];                                         //
  }                                                                           //
  if ( mqtt_in_data == "0" )                                                  //
  {                                                                           //
    digitalWrite  ( relay_off, OFF );                                         //
    delay         ( 5 );                                                      //
    digitalWrite  ( relay_off, ON  );                                         //
    switch1       = OFF;                                                      //
    digitalWrite  ( switch1, 0 );                                             //
  }                                                                           //
  if ( mqtt_in_data == "1" )                                                  //
  {                                                                           //
    digitalWrite  ( relay_on, OFF );                                          //
    delay         ( 5 );                                                      //
    digitalWrite  ( relay_on, ON  );                                          //
    switch1       = ON;                                                       //
  }                                                                           //
}                                                                             // 
// ***********************************************************************************************************************
