// ***********************************************************************************************************************
// ** File:         ntp.h                                                                                               **
// ** Title:        Client for Network Time Protocol                                                                    **
// ** CPU:          ESP-8266                                                                                            **
// ** Created:      23.04.2016                                                                                          **
// ** Revised:      23.04.2016                                                                                          **
// ** Version:      1.0                                                                                                 **
// ** Note:                                                                                                             **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// ***********************************************************************************************************************

// COMPILER INFORMATION **************************************************************************************************
#ifndef NTP_H                                                               //
#define NTP_H                                                               //
// ***********************************************************************************************************************

// INCLUDES **************************************************************************************************************
#include "global.h"                                                         // 
#include <stdlib.h>                                                         // 
#include <stdio.h>                                                          // 
#include <string.h>                                                         // 
// ***********************************************************************************************************************

// DEFINES ***************************************************************************************************************
#define FIRSTYEAR               1900                                        // start year
#define FIRSTDAY                1                                           // 0 = Sunday
#define NTP_PACKET_SIZE         48                                          // 
#define NTP_HEADER_LENGTH       48                                          // 
#define NTP_NO_ANSWER           0                                           // 
#define NTP_ANSWER              1                                           // 
#define NTP_REQ_PENDING         2                                           // 
#define NTP_PORT                htons(123)                                  // Port der fuer NTP Zeitabfragen genutzt wird
#define DAYPERWEEK              7                                           //
#define GMT                     +1                                          // 
#define TIMEZONE                GMT * 3600                                  //
#define TIMEOUT                 10                                          //
#define SZ_SET                  0                                           // Sommerzeit Flag
#define SZ_NOT_SET              !SZ_SET                                     // Sommerzeit Flag
// ***********************************************************************************************************************

// CONSTANTS *************************************************************************************************************
extern const char               *Month_to_string[12];                       //
extern const char               *Day_to_wdays[7];                           //
// ***********************************************************************************************************************

// VARIABLES *************************************************************************************************************
extern unsigned long    timestamp;                                          //
extern unsigned char    second;                                             // 
extern unsigned char    minute;                                             // 
extern unsigned char    hour;                                               // 
extern unsigned char    wday;                                               // 
extern unsigned char    day;                                                // Tagesspeicher
extern unsigned char    month;                                              // Monatsspeicher
extern unsigned int     year;                                               // Jahresspeicher
extern unsigned char    ntp_flag;                                             // 
// ***********************************************************************************************************************

// NTP Struct ************************************************************************************************************
struct NTP_header                                                           //
{                                                                           //
    unsigned char       NTP_Flags;                                          // 1 byte
    unsigned char       NTP_Clock_Stratum;                                  // 1 byte
    unsigned char       NTP_Polling_Interval;                               // 1 byte
    unsigned char       NTP_Clock_Precision;                                // 1 byte
    unsigned long       NTP_Root_Delay;                                     // 4 byte
    unsigned long       NTP_Root_Dispersion;                                // 4 byte
    unsigned long       NTP_Reference_ID;                                   // 4 byte
    unsigned long long  NTP_Reference_Timest;                               // 8 byte
    unsigned long long  NTP_Origin_Timest;                                  // 8 byte
    unsigned long long  NTP_Receive_Timest;                                 // 8 byte
    unsigned long long  NTP_Transmit_Timest;                                // 8 byte = 48 Bytes  
};                                                                          //
// ***********************************************************************************************************************
#endif
