// ***********************************************************************************************************************
// ** File:         ntp.h                                                                                               **
// ** Title:        Client for Network Time Protocol                                                                    **
// ** CPU:          ESP-8266                                                                                            **
// ** Created:      23.04.2016                                                                                          **
// ** Revised:      23.04.2016                                                                                          **
// ** Version:      1.0                                                                                                 **
// ** Note:                                                                                                             **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// ***********************************************************************************************************************

// INCLUDES **************************************************************************************************************
#include "ntp.h"                                                            // 
// ***********************************************************************************************************************

// ***********************************************************************************************************************
WiFiUDP         ntp;                                                        // 
char            ntp_buffer[ NTP_PACKET_SIZE ];                              // 
unsigned long   timestamp   = 3671721860;                                   //
unsigned char   second      = 0;                                            // 
unsigned char   minute      = 0;                                            // 
unsigned char   hour        = 0;                                            // 
unsigned char   wday        = 0;                                            // 
unsigned char   day         = 1;                                            // Tagesspeicher
unsigned char   month       = 1;                                            // Monatsspeicher
unsigned int    year        = 1900;                                         // Jahresspeicher
unsigned char   ntp_flag    = NTP_NO_ANSWER;                                  // 
// ***********************************************************************************************************************

// CONSTANTS *************************************************************************************************************
const char *Month_to_string[12] =                                           // 
{                                                                           // 
    "Jan", "Feb", "Mae", "Apr", "Mai", "Jun",                               // 
    "Jul", "Aug", "Sep", "Okt", "Nov", "Dez"                                // 
};                                                                          // 
const char *Day_to_wdays[7] =                                               // 
{                                                                           // 
    "So", "Mo", "Di", "Mi", "Do", "Fr", "Sa"                                // 
};                                                                          // 
unsigned char Month_to_days[12] =                                           // 
{                                                                           // 
    31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31                          // 
};                                                                          // 
// ***********************************************************************************************************************

// FUNCTION **************************************************************************************************************
void udp_start ( void )                                                     // 
{                                                                           // 
  ntp_flag    = NTP_NO_ANSWER;                                                // 
  ntp.begin ( 123 );                                                        // 
}                                                                           // 
// ***********************************************************************************************************************

// FUNCTION **************************************************************************************************************
void ntp_receiver ( void )                                                  // 
{                                                                           // 
  struct NTP_header   *NTP_packet     = ( struct NTP_header * )             // Strukt wird an position NTP_HEADER_START auf 
                                        &ntp_buffer[ 0 ];                   // den enc Buffer angewendet
  unsigned long ntp_timestamp = 0x00000000;                                 // Hilfsvariable fuer empfangenen Zeitstempel
  unsigned int packetSize     = ntp.parsePacket ();                         // 
  unsigned int remote_port    = ntp.remotePort  ();                         // 
  if ( packetSize )                                                         // We've received a packet, read the data from it
  {                                                                         // 
    if ( remote_port == 123 )                                               // 
    {                                                                       // 
      ntp.read        ( ntp_buffer, NTP_PACKET_SIZE );                      // read the packet into the buffer
      ntp_timestamp = htonl( ( unsigned long )( NTP_packet->NTP_Transmit_Timest ) );// Zeitstempel aus empfangenen Paket holen
      timestamp = ntp_timestamp;                                            // internen Zaehler mit Zeitstempel synchronisieren
      timestamp_decode();                                                   // 
      ntp_flag    = NTP_ANSWER;                                               // 
    }                                                                       // 
  }                                                                         // 
}                                                                           // 
// ***********************************************************************************************************************

// FUNCTIONS *************************************************************************************************************
void send_ntp_packet ( )                                                    // Baut einen gueltigen NTP Header
{                                                                           // 
    IPAddress timeServerIP;                                                 // time.nist.gov NTP server address
    //const char* ntpServerName = "fritz.box";                              // 
    //const char* ntpServerName = "time.nist.gov";                          // 
    WiFi.hostByName( ntp_server, timeServerIP );                            // 
    char   ntp_buffer[ NTP_PACKET_SIZE ];                                   // 
    struct NTP_header * NTP_packet      = ( struct NTP_header * )           // Strukt wird an position NTP_HEADER_START auf  
                                          &ntp_buffer[ 0 ];                 // den enc Buffer angewendet
    NTP_packet->NTP_Flags               = 0xdb;                             // 
    NTP_packet->NTP_Clock_Stratum       = 0x00;                             // 8-bit integer representing the stratum
    NTP_packet->NTP_Polling_Interval    = 0x0a;                             // 8-bit signed integer representing the maximum interval between successive messages
    NTP_packet->NTP_Clock_Precision     = 0xfa;                             // 8-bit signed integer representing the precision of the system clock
    NTP_packet->NTP_Root_Delay          = 0x00000000;                       // Total round-trip delay to the reference clock
    NTP_packet->NTP_Root_Dispersion     = 0x00000000;                       // Total dispersion to the reference clock
    NTP_packet->NTP_Reference_ID        = 0x00000000;                       // 32-bit code identifying the particular server
    NTP_packet->NTP_Reference_Timest    = 0x0000000000000000;               // Time when the system clock was last set or corrected
    NTP_packet->NTP_Origin_Timest       = 0x0000000000000000;               // Time at the client when the request departed for the server
    NTP_packet->NTP_Receive_Timest      = 0x0000000000000000;               // Time at the server when the request arrived from the client
    NTP_packet->NTP_Transmit_Timest     = 0x0000000000000000;               // Time at the server when the response left for the client
    ntp.beginPacket ( timeServerIP, 123 );                                  // NTP requests are to port 123
    ntp.write       ( ntp_buffer, NTP_PACKET_SIZE );                        // 
    ntp.endPacket   ( );                                                    // 
    ntp_flag    = NTP_REQ_PENDING;                                  // 
}                                                                           // 
// ***********************************************************************************************************************

// ***********************************************************************************************************************
void summertime( void )                                                     // Check for Daylight saving Time
{                                                                           //
    unsigned char temp_hour, temp_day, temp_wday, temp_month;               // locals for faster access
    temp_hour   = hour;                                                     //
    temp_day    = day;                                                      //
    temp_wday   = wday;                                                     //
    temp_month  = month;                                                    //
    if( temp_month < 3 || temp_month > 10 )                                 // month 1, 2, 11, 12
    {                                                                       //
        return;                                                             // -> Winter
    }                                                                       //
    if( temp_day - temp_wday >= 25 && (temp_wday || temp_hour >= 2) )       // after last Sunday 2:00
    {                                                                       //
        if( temp_month == 10 )                                              // October -> Winter
        {                                                                   //
            return;                                                         //
        }                                                                   //
    }                                                                       //
    else                                                                    //
    {                                                                       // before last Sunday 2:00
        if( temp_month == 3 )                                               // March -> Winter
        {                                                                   //
            return;                                                         //
        }                                                                   //
    }                                                                       //
    temp_hour++;                                                            // add one hour
    if( temp_hour == 24 )                                                   // next day
    {                                                                       //
        temp_hour = 0;                                                      //
        temp_wday++;                                                        // next weekday
    temp_day++;                                                             // 
        if( temp_wday == 7 )                                                //
        {                                                                   //
            temp_wday = 0;                                                  //
        }                                                                   //
        if( temp_day > Month_to_days[temp_month-1] )                        // next month
        {                                                                   //
            temp_day = 1;                                                   //
            temp_month++;                                                   //
        }                                                                   //
    }                                                                       //
    month   = temp_month;                                                   //
    hour    = temp_hour;                                                    //
    day     = temp_day;                                                     //
    wday    = temp_wday;                                                    // 
}                                                                           //
// ***********************************************************************************************************************

// ***********************************************************************************************************************
void timestamp_decode ( void )                                              // Decode the NTP-Timestamp
{                                                                           //
    unsigned short  temp_day;                                               //
    unsigned char   temp_year;                                              //
    unsigned short  temp_dayofyear;                                         //
    unsigned char   temp_leap400;                                           //
    unsigned char   temp_month;                                             //
    unsigned long   resc_time = 0;                                          //
    resc_time       = timestamp;                                            // Zeitstempel sichern
    timestamp       = timestamp + TIMEZONE;                                 // Zeitinformation + Timezone
    second          = timestamp % 60;                                       //
    timestamp       /= 60;                                                  //
    minute          = timestamp % 60;                                       //
    timestamp       /= 60;                                                  //
    hour            = timestamp % 24;                                       //
    temp_day        = timestamp / 24;                                       //
    wday            = (temp_day + FIRSTDAY) % 7;                            // weekday
    temp_year       = FIRSTYEAR % 100;                                      // 0..99
    temp_leap400    = 4 - ((FIRSTYEAR - 1) / 100 & 3);                      // 4, 3, 2, 1
    while( 1 )                                                              //
    {                                                                       //
        temp_dayofyear = 365;                                               //
        if( (temp_year & 3) == 0 )                                          //
        {                                                                   //
            temp_dayofyear = 366;                                           // leap year
            if( temp_year == 0 || temp_year == 100 || temp_year == 200 )    // 100 year exception
            {                                                               //
                if( --temp_leap400 )                                        // 400 year exception
                {                                                           //
                    temp_dayofyear = 365;                                   //
                }                                                           //
            }                                                               //
        }                                                                   //
        if( temp_day < temp_dayofyear )                                     //
        {                                                                   //
            break;                                                          //
        }                                                                   //
        temp_day -= temp_dayofyear;                                         //
        temp_year++;                                                        // 00..136 / 99..235
    }                                                                       //
    year = temp_year + FIRSTYEAR / 100 * 100;                               // + century
    if( temp_dayofyear & 1 && temp_day > 58 )                               // no leap year and after 28.2.
    {                                                                       //
        temp_day++;                                                         // skip 29.2.
    }                                                                       //
    for( temp_month = 1; temp_day >= Month_to_days[temp_month-1]; temp_month++ )//
    {                                                                       //
        temp_day -= Month_to_days[temp_month-1];                            //
    }                                                                       //
    month = temp_month;                                                     // 1..12
    day = temp_day + 1;                                                     // 1..31
    timestamp = resc_time;                                                  // Zeitstempel zurueckschreiben
    summertime( );                                                          //
}                                                                           //
// ***********************************************************************************************************************

